#ifndef	_SYS_LOCKMGRNG_H_
#define	_SYS_LOCKMGRNG_H_

#include <sys/_lock.h>
#include <sys/_lockmgrng.h>

#define	LN_SHARED			0x01
#define	LN_SHARED_WAITERS		0x02
#define	LN_EXCLUSIVE_WAITERS		0x04
#define	LN_UPGRADE_WAITERS		0x08
#define	LN_DRAIN_WAITERS		0x10
#define	LN_EXCLDRAIN_WAITERS						\
	(LN_EXCLUSIVE_WAITERS | LN_DRAIN_WAITERS)
#define	LN_FLAGMASK							\
	(LN_SHARED | LN_SHARED_WAITERS | LN_EXCLUSIVE_WAITERS |		\
	LN_UPGRADE_WAITERS)

#define	LN_HOLDER(x)			((x) & ~LN_FLAGMASK)
#define	LN_SHARERS_SHIFT		5
#define	LN_SHARERS(x)			(LN_HOLDER(x) >> LN_SHARERS_SHIFT)
#define	LN_SHARERS_LOCK(x)		((x) << LN_SHARERS_SHIFT | LN_SHARED)
#define	LN_ONE_SHARER			(1 << LN_SHARERS_SHIFT)
#define	LN_UNHELD			LN_SHARERS_LOCK(0)

#ifdef _KERNEL

struct mtx;

/*
 * Function prototipes.  Routines that start with an underscore are not part
 * of the public interface and might be wrappered with a macro.
 */
void	ln_init(struct lockng *ln, int timo, int pri, const char *wmesg);
void	ln_destroy(struct lockng *ln);
int	_ln_slock(struct lockng *ln, uint32_t flags, struct mtx *ilock,
	    struct thread *td, const char *file, int line);
int	_ln_xlock(struct lockng *ln, uint32_t flags, struct mtx *ilock,
	    struct thread *td, const char *file, int line);
void	_ln_sunlock(struct lockng *ln, const char *file, int line);
void	_ln_xunlock(struct lockng *ln, struct thread *td, const char *file,
	    int line);
void	_ln_downgrade(struct lockng *ln, struct thread *td, const char *file,
	    int line);
int	_ln_upgrade(struct lockng *ln, uint32_t flags, struct mtx *ilock,
	    struct thread *td, const char *file, int line);
int	_ln_drain(struct lockng *ln, uint32_t flags, struct mtx *ilock,
	    struct thread *td, const char *file, int line);
void	_ln_undrain(struct lockng *ln, struct thread *td, const char *file,
	    int line);

/*
 * Public interface for lock operations.
 */
#define	ln_slock(ln, flags, ilock, td)					\
	_ln_slock((ln), (flags), (ilock), (td), LOCK_FILE, LOCK_LINE)
#define	ln_xlock(ln, flags, ilock, td)					\
	_ln_xlock((ln), (flags), (ilock), (td), LOCK_FILE, LOCK_LINE)
#define	ln_sunlock(ln)		_ln_sunlock((ln), LOCK_FILE, LOCK_LINE)
#define	ln_xunlock(ln, td)	_ln_xunlock((ln), (td), LOCK_FILE, LOCK_LINE)
#define	ln_downgrade(ln, td)	_ln_downgrade((ln), (td), LOCK_FILE, LOCK_LINE)
#define	ln_upgrade(ln, flags, ilock, td)				\
	_ln_upgrade((ln), (flags), (ilock), (td), LOCK_FILE, LOCK_LINE)
#define	ln_drain(ln, flags, ilock, td)					\
	_ln_drain((ln), (flags), (ilock), (td), LOCK_FILE, LOCK_LINE)
#define	ln_undrain(ln, td)	_ln_undrain((ln), (td), LOCK_FILE, LOCK_LINE)

/*
 * Internal flags.
 */
#define	LK_EXCLUPGRADE	0x01
#define	LK_INTERLOCK	0x02
#define	LK_NOWAIT	0x04
#define	LK_SLEEPFAIL	0x08

/*
 * KERNPROC. yes, this is a fake value :)
 */
#define	LK_KERNPROC	0xdeadc0de

#endif /* _KERNEL */

#endif /* !_SYS_LOCKMGRNG_H_ */
